@@ -5,8 +5,30 @@ use crate::interrupt::test_int;
 use crate::num::{Base, FormattingStyle, Number};
 use crate::scope::Scope;
 use crate::value::{ApplyMulHandling, BuiltInFunction, Value};
+use std::fmt;
 use std::sync::Arc;
 
+#[derive(Copy, Clone, Debug, PartialEq, Eq)]
+pub(crate) enum Bop {
+    Plus,
+    Minus,
+    Mul,
+    Div,
+    Mod,
+}
+
+impl fmt::Display for Bop {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        match self {
+            Self::Plus => write!(f, "+"),
+            Self::Minus => write!(f, "-"),
+            Self::Mul => write!(f, "*"),
+            Self::Div => write!(f, "/"),
+            Self::Mod => write!(f, " mod "),
+        }
+    }
+}
+
 #[derive(Clone, Debug)]
 pub(crate) enum Expr {
     Literal(Value),
@@ -16,12 +38,7 @@ pub(crate) enum Expr {
     UnaryPlus(Box<Expr>),
     UnaryDiv(Box<Expr>),
     Factorial(Box<Expr>),
-    Add(Box<Expr>, Box<Expr>),
-    ImplicitAdd(Box<Expr>, Box<Expr>),
-    Sub(Box<Expr>, Box<Expr>),
-    Mul(Box<Expr>, Box<Expr>),
-    Div(Box<Expr>, Box<Expr>),
-    Mod(Box<Expr>, Box<Expr>),
+    Bop(Bop, Box<Expr>, Box<Expr>),
     Pow(Box<Expr>, Box<Expr>),
     // Call a function or multiply the expressions
     Apply(Box<Expr>, Box<Expr>),
@@ -54,13 +71,9 @@ impl<'a> Expr {
             Self::UnaryPlus(x) => format!("(+{})", x.format(ctx, int)?),
             Self::UnaryDiv(x) => format!("(/{})", x.format(ctx, int)?),
             Self::Factorial(x) => format!("{}!", x.format(ctx, int)?),
-            Self::Add(a, b) | Self::ImplicitAdd(a, b) => {
-                format!("({}+{})", a.format(ctx, int)?, b.format(ctx, int)?)
+            Self::Bop(op, a, b) => {
+                format!("({}{}{})", a.format(ctx, int)?, op, b.format(ctx, int)?)
             }
-            Self::Sub(a, b) => format!("({}-{})", a.format(ctx, int)?, b.format(ctx, int)?),
-            Self::Mul(a, b) => format!("({}*{})", a.format(ctx, int)?, b.format(ctx, int)?),
-            Self::Div(a, b) => format!("({}/{})", a.format(ctx, int)?, b.format(ctx, int)?),
-            Self::Mod(a, b) => format!("({} mod {})", a.format(ctx, int)?, b.format(ctx, int)?),
             Self::Pow(a, b) => format!("({}^{})", a.format(ctx, int)?, b.format(ctx, int)?),
             Self::Apply(a, b) => format!("({} ({}))", a.format(ctx, int)?, b.format(ctx, int)?),
             Self::ApplyFunctionCall(a, b) | Self::ApplyMul(a, b) => {
@@ -126,10 +139,8 @@ pub(crate) fn evaluate<I: Interrupt>(
         Expr::Factorial(x) => {
             eval!(*x)?.handle_num(|x| x.factorial(int), Expr::Factorial, scope)?
         }
-        Expr::Add(a, b) | Expr::ImplicitAdd(a, b) => {
-            evaluate_add(eval!(*a)?, eval!(*b)?, scope, int)?
-        }
-        Expr::Sub(a, b) => {
+        Expr::Bop(Bop::Plus, a, b) => evaluate_add(eval!(*a)?, eval!(*b)?, scope, int)?,
+        Expr::Bop(Bop::Minus, a, b) => {
             let a = eval!(*a)?;
             match a {
                 Value::Num(a) => Value::Num(Box::new(a.sub(eval!(*b)?.expect_num()?, int)?)),
@@ -143,11 +154,11 @@ pub(crate) fn evaluate<I: Interrupt>(
                 _ => return Err("invalid operands for subtraction".to_string().into()),
             }
         }
-        Expr::Mul(a, b) => eval!(*a)?.handle_two_nums(
+        Expr::Bop(bop, a, b) => eval!(*a)?.handle_two_nums(
             eval!(*b)?,
-            |a, b| a.mul(b, int),
-            |a| |f| Expr::Mul(f, Box::new(Expr::Literal(Value::Num(Box::new(a))))),
-            |a| |f| Expr::Mul(Box::new(Expr::Literal(Value::Num(Box::new(a)))), f),
+            |a, b| a.bop(bop, b, int),
+            |a| |f| Expr::Bop(bop, f, Box::new(Expr::Literal(Value::Num(Box::new(a))))),
+            |a| |f| Expr::Bop(bop, Box::new(Expr::Literal(Value::Num(Box::new(a)))), f),
             scope,
         )?,
         Expr::Apply(a, b) | Expr::ApplyMul(a, b) => {
@@ -159,20 +170,6 @@ pub(crate) fn evaluate<I: Interrupt>(
             }
             eval!(*a)?.apply(*b, ApplyMulHandling::Both, scope, context, int)?
         }
-        Expr::Div(a, b) => eval!(*a)?.handle_two_nums(
-            eval!(*b)?,
-            |a, b| a.div(b, int),
-            |a| |f| Expr::Div(f, Box::new(Expr::Literal(Value::Num(Box::new(a))))),
-            |a| |f| Expr::Div(Box::new(Expr::Literal(Value::Num(Box::new(a)))), f),
-            scope,
-        )?,
-        Expr::Mod(a, b) => eval!(*a)?.handle_two_nums(
-            eval!(*b)?,
-            |a, b| a.modulo(b, int),
-            |a| |f| Expr::Mod(f, Box::new(Expr::Literal(Value::Num(Box::new(a))))),
-            |a| |f| Expr::Mod(Box::new(Expr::Literal(Value::Num(Box::new(a)))), f),
-            scope,
-        )?,
         Expr::Pow(a, b) => {
             let lhs = eval!(*a)?;
             if should_compute_inverse(&*b) {
@@ -230,21 +227,29 @@ fn evaluate_add<I: Interrupt>(
             Value::String(format!("{}{}", a.as_ref(), b.as_ref()).into())
         }
         (Value::BuiltInFunction(f), Value::Num(a)) => f.wrap_with_expr(
-            |f| Expr::Add(f, Box::new(Expr::Literal(Value::Num(a)))),
+            |f| Expr::Bop(Bop::Plus, f, Box::new(Expr::Literal(Value::Num(a)))),
             scope,
         ),
         (Value::Num(a), Value::BuiltInFunction(f)) => f.wrap_with_expr(
-            |f| Expr::Add(Box::new(Expr::Literal(Value::Num(a))), f),
+            |f| Expr::Bop(Bop::Plus, Box::new(Expr::Literal(Value::Num(a))), f),
             scope,
         ),
         (Value::Fn(param, expr, scope), Value::Num(a)) => Value::Fn(
             param,
-            Box::new(Expr::Add(expr, Box::new(Expr::Literal(Value::Num(a))))),
+            Box::new(Expr::Bop(
+                Bop::Plus,
+                expr,
+                Box::new(Expr::Literal(Value::Num(a))),
+            )),
             scope,
         ),
         (Value::Num(a), Value::Fn(param, expr, scope)) => Value::Fn(
             param,
-            Box::new(Expr::Add(Box::new(Expr::Literal(Value::Num(a))), expr)),
+            Box::new(Expr::Bop(
+                Bop::Plus,
+                Box::new(Expr::Literal(Value::Num(a))),
+                expr,
+            )),
             scope,
         ),
         (a, b) => return a.add_dyn(b),
