@@ -1,3 +1,4 @@
+use crate::value::ValueTrait;
 use std::fmt;
 
 #[derive(Clone, Copy, PartialEq, Eq)]
@@ -31,3 +32,16 @@ impl fmt::Display for DayOfWeek {
         write!(f, "{:?}", self)
     }
 }
+
+impl ValueTrait for DayOfWeek {
+    fn type_name(&self) -> &'static str {
+        "day of week"
+    }
+
+    fn format(&self, _indent: usize, spans: &mut Vec<crate::Span>) {
+        spans.push(crate::Span {
+            string: self.to_string(),
+            kind: crate::SpanKind::Date,
+        });
+    }
+}
