@@ -1,4 +1,4 @@
-use crate::ast::Expr;
+use crate::ast::{Bop, Expr};
 use crate::lexer::{Symbol, Token};
 use crate::value::Value;
 use std::fmt;
@@ -220,7 +220,7 @@ fn parse_mixed_fraction<'a>(input: &'a [Token], lhs: &Expr) -> ParseResult<'a> {
                 return Err(ParseError::InvalidMixedFraction);
             }
         }
-        Expr::Mul(a, b) => match &**b {
+        Expr::Bop(Bop::Mul, a, b) => match &**b {
             Expr::Literal(Value::Num(_)) => (true, &**b, Some(&**a)),
             Expr::UnaryMinus(x) => {
                 if let Expr::Literal(Value::Num(_)) = &**x {
@@ -244,14 +244,18 @@ fn parse_mixed_fraction<'a>(input: &'a [Token], lhs: &Expr) -> ParseResult<'a> {
     } else {
         return Err(ParseError::InvalidMixedFraction);
     }
-    let rhs = Box::new(Expr::Div(Box::new(rhs_top), Box::new(rhs_bottom)));
+    let rhs = Box::new(Expr::Bop(Bop::Div, Box::new(rhs_top), Box::new(rhs_bottom)));
     let mixed_fraction = if positive {
-        Expr::Add(Box::new(lhs.clone()), rhs)
+        Expr::Bop(Bop::Plus, Box::new(lhs.clone()), rhs)
     } else {
-        Expr::Sub(Box::new(lhs.clone()), rhs)
+        Expr::Bop(Bop::Minus, Box::new(lhs.clone()), rhs)
     };
     let mixed_fraction = other_factor.map_or(mixed_fraction.clone(), |other_factor| {
-        Expr::Mul(Box::new(other_factor.clone()), Box::new(mixed_fraction))
+        Expr::Bop(
+            Bop::Mul,
+            Box::new(other_factor.clone()),
+            Box::new(mixed_fraction),
+        )
     });
     Ok((mixed_fraction, input))
 }
@@ -278,13 +282,13 @@ fn parse_multiplicative(input: &[Token]) -> ParseResult<'_> {
     let (mut res, mut input) = parse_power(input, true)?;
     loop {
         if let Ok((term, remaining)) = parse_multiplication_cont(input) {
-            res = Expr::Mul(Box::new(res.clone()), Box::new(term));
+            res = Expr::Bop(Bop::Mul, Box::new(res.clone()), Box::new(term));
             input = remaining;
         } else if let Ok((term, remaining)) = parse_division_cont(input) {
-            res = Expr::Div(Box::new(res.clone()), Box::new(term));
+            res = Expr::Bop(Bop::Div, Box::new(res.clone()), Box::new(term));
             input = remaining;
         } else if let Ok((term, remaining)) = parse_modulo_cont(input) {
-            res = Expr::Mod(Box::new(res.clone()), Box::new(term));
+            res = Expr::Bop(Bop::Mod, Box::new(res.clone()), Box::new(term));
             input = remaining;
         } else if let Ok((new_res, remaining)) = parse_mixed_fraction(input, &res) {
             res = new_res;
@@ -303,9 +307,13 @@ fn parse_implicit_addition(input: &[Token]) -> ParseResult<'_> {
     let (res, input) = parse_multiplicative(input)?;
     if let Ok((rhs, remaining)) = parse_implicit_addition(input) {
         // n i n i, n i i n i i, etc. (n: number literal, i: identifier)
-        if let (Expr::ApplyMul(_, _), Expr::ApplyMul(_, _) | Expr::ImplicitAdd(_, _)) = (&res, &rhs)
+        if let (Expr::ApplyMul(_, _), Expr::ApplyMul(_, _) | Expr::Bop(Bop::Plus, _, _)) =
+            (&res, &rhs)
         {
-            return Ok((Expr::ImplicitAdd(Box::new(res), Box::new(rhs)), remaining));
+            return Ok((
+                Expr::Bop(Bop::Plus, Box::new(res), Box::new(rhs)),
+                remaining,
+            ));
         };
     }
     Ok((res, input))
@@ -333,10 +341,10 @@ fn parse_additive(input: &[Token]) -> ParseResult<'_> {
     let (mut res, mut input) = parse_implicit_addition(input)?;
     loop {
         if let Ok((term, remaining)) = parse_addition_cont(input) {
-            res = Expr::Add(Box::new(res), Box::new(term));
+            res = Expr::Bop(Bop::Plus, Box::new(res), Box::new(term));
             input = remaining;
         } else if let Ok((term, remaining)) = parse_subtraction_cont(input) {
-            res = Expr::Sub(Box::new(res), Box::new(term));
+            res = Expr::Bop(Bop::Minus, Box::new(res), Box::new(term));
             input = remaining;
         } else if let Ok((term, remaining)) = parse_to_cont(input) {
             res = Expr::As(Box::new(res), Box::new(term));
