@@ -1,3 +1,4 @@
+use crate::ast::Bop;
 use crate::error::{FendError, Interrupt};
 use crate::num::{Base, FormattingStyle, Number};
 use crate::scope::Scope;
@@ -264,7 +265,7 @@ impl Value {
                 let n2 = n.clone();
                 other.handle_num(
                     |x| n.mul(x, int),
-                    |x| Expr::Mul(Box::new(Expr::Literal(Self::Num(n2))), x),
+                    |x| Expr::Bop(Bop::Mul, Box::new(Expr::Literal(Self::Num(n2))), x),
                     scope,
                 )?
             }
@@ -314,7 +315,6 @@ impl Value {
             BuiltInFunction::Log2 => arg.expect_num()?.log2(int)?,
             BuiltInFunction::Log10 => arg.expect_num()?.log10(int)?,
             BuiltInFunction::Base => {
-                use std::convert::TryInto;
                 let n: u8 = arg
                     .expect_num()?
                     .try_as_usize(int)?
